# Generated by Django 3.0.7 on 2020-06-14 11:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('avishan', '0011_auto_20200526_1740'),
    ]

    operations = [
        migrations.DeleteModel(
            name='RequestTrackExecInfo',
        ),
    ]
