# todo 0.2.3: can we count db calls with decorator?
# todo 0.2.2: save user device on every request
# todo 0.2.1: readme edit to latest
# todo 0.2.2: app check function change to class. to use its parameters.
# todo 0.2.2: AvishanWarning with color and text and usage
# todo 0.2.3: execution time avishan_calculate decorator
# todo 0.2.2: video model

"""avishan_config file"""
# todo 0.2.2: create avishan_config.py file to each app: to always run it at startup
# todo 0.2.2: create avishan_config.py 'check' function and 'AvishanConfig' class
# todo 0.2.1: add avishan app to INSTALLED_APPS
# todo 0.2.1: add avishan middlewares to list
# todo 0.2.1: setup datetime settings & use_tz
# todo 0.2.1: setup static & media settings
# todo 0.2.1: handle cors headers
# todo 0.2.1: add and comment database supported settings
# todo 0.2.1: move secret_key to config file
# todo 0.2.1: ask for project name and add it to settings.py. to put avishan_config in project folder not root
